/**
 * dep: dsbridge
 */

if (typeof Object.assign != "function") {
  // Must be writable: true, enumerable: false, configurable: true
  Object.defineProperty(Object, "assign", {
    value: function assign(target, varArgs) {
      // .length of function is 2
      "use strict";
      if (target == null) {
        // TypeError if undefined or null
        throw new TypeError("Cannot convert undefined or null to object");
      }

      let to = Object(target);

      for (var index = 1; index < arguments.length; index++) {
        var nextSource = arguments[index];

        if (nextSource != null) {
          // Skip over if undefined or null
          for (let nextKey in nextSource) {
            // Avoid bugs when hasOwnProperty is shadowed
            if (Object.prototype.hasOwnProperty.call(nextSource, nextKey)) {
              to[nextKey] = nextSource[nextKey];
            }
          }
        }
      }
      return to;
    },
    writable: true,
    configurable: true
  });
}

(function(global) {
  var IS_QUICKAPP_MODE =
    window.navigator.userAgent.indexOf("hap/") > -1 && !!window["system"];
  var native = { IS_QUICKAPP_MODE: IS_QUICKAPP_MODE };
  var _actionsA = [
      "authInfo",
      "getPhone",
      "getUid",
      "startWithdrawActivity",
      "bindSuccess",
      "unBindSuccess"
    ],
    _actionsB = [
      "callPhone",
      "refreshAuth",
      "login",
      "share",
      "close",
      "back",
      "openUrl",
      "recharge",
      "service",
      "withdraw",
      "tradeTab"
    ],
    index = _actionsA.length - 1,
    actions = _actionsA.concat(_actionsB);

  if (IS_QUICKAPP_MODE) {
    var uuid = 0,
      callbackPool = {};
    system.onmessage = function(msg) {
      try {
        msg = JSON.parse(msg);
        var callback = callbackPool[msg.uuid];
        if (typeof callback === "function") {
          callback(msg.data);
          delete callbackPool[msg.uuid];
        }
      } catch (ex) {}
    };

    actions.forEach(function(action, i) {
      native[action] = function(arg, callback) {
        if (typeof arg === "function") {
          callback = arg;
          arg = null;
        }

        system.postMessage(
          JSON.stringify(
            Object.assign(
              {
                action: action,
                url: "",
                uuid: uuid,
                callback: typeof callback === "function"
              },
              arg
            )
          )
        );
        if (typeof callback === "function") {
          callbackPool[uuid] = callback;
        }
        uuid++;
        return true;
      };
    });
    native.hasNativeMethod = function(methodName) {
      return true;
    };
  } else {
    actions.forEach(function(action, i) {
      var command = i > index ? "jsApiTodoNative" : action;
      native[action] = function(arg, callback) {
        if (typeof arg === "function") {
          callback = arg;
          arg = null;
        }
        try {
          var value = dsBridge.call(
            command,
            Object.assign({ action: action, url: "" }, { arg: arg })
          );
          if (typeof callback === "function") {
            callback(value);
            return value;
          }
          return true;
        } catch (error) {
          callback(error);
          return false;
        }
      };
    });
    native.hasNativeMethod = function(methodName) {
      return dsBridge.hasNativeMethod(methodName);
    };
  }

  global.native = native;
})(this);
